#' GetMissingnessSummary
#'
#' Get missingness analysis for an object. Imported from VIM packages.
#'
#' @param x Data to be analysed for missingness. Required.
#' @param delimiter a character-vector to distinguish between variables and
#'   imputation-indices for imputed variables. Default: NULL.
#'
#' @return list object
#'
#' @examples
#' \dontrun{
#' GetMissingnessSummary(x)
#' }
#'
#' @export
GetMissingnessSummary <- function(x, delimiter = NULL)
{
  countNA <- function(x) sum(is.na(x))
  defaultNames <- function(p) paste("Var", 1:p, sep = "")
  getEscapeChars <- function() c("", "_", "__", "-", " ", "/", "\\", ",")
  countImp <- function(x, delimiter, imp_var) {
    escape <- getEscapeChars()
    for (i in 1:length(escape)) {
      indexh <- (paste(colnames(x), delimiter, sep = escape[i])) %in%
        colnames(imp_var)
      if (any(indexh)) {
        escape <- escape[i]
        break
      }
    }
    tmp <- integer(ncol(x))
    names(tmp) <- colnames(x)
    for (i in 1:ncol(x)) {
      tmp[i] <- ifelse(indexh[i], length(which(imp_var[, paste(colnames(x)[i],
                                                               delimiter, sep = escape)])), 0)
    }
    tmp
  }

  isImp <- function(x, pos, delimiter, imp_var, selection = c("none", "any", "all")) {
    selection <- match.arg(selection)
    escape <- getEscapeChars()
    if (is.null(dim(x)) || is.null(dim(imp_var)))
      list(misspos = imp_var, impp = TRUE, missh = rep(FALSE,
                                                       NROW(x)))
    else {
      for (i in 1:length(escape)) {
        indexp <- colnames(imp_var) %in% paste(colnames(x)[pos],
                                               delimiter, sep = escape[i])
        if (any(indexp))
          break
      }
      if (any(indexp)) {
        misspos <- imp_var[, indexp]
        impp <- TRUE
        imp_var <- imp_var[, !indexp, drop = FALSE]
      }
      else {
        misspos <- rep(FALSE, nrow(x))
        impp <- FALSE
      }
      for (i in 1:length(escape)) {
        indexh <- (paste(colnames(x), delimiter, sep = escape[i])) %in%
          colnames(imp_var)
        if (any(indexh)) {
          escape <- escape[i]
          break
        }
      }
      if (any(indexh)) {
        index <- which(indexh)
        tmp <- matrix(nrow = nrow(x), ncol = length(index))
        for (i in 1:length(index)) {
          tmp[, i] <- imp_var[, paste(colnames(x)[index[i]],
                                      delimiter, sep = escape)]
        }
        if (length(index) > 1 && selection != "none") {
          missh <- apply(tmp, 1, function(tmp) eval(call(selection,
                                                         tmp)))
        }
        else {
          missh <- tmp
          colnames(missh) <- colnames(x[, indexh])
        }
      }
      else {
        missh <- rep(FALSE, nrow(x))
      }
      list(misspos = misspos, impp = impp, missh = missh)
    }
  }

  imputed <- FALSE
  if (is.null(dim(x))) {
    n <- length(x)
    imp <- FALSE
    cn <- defaultNames(1)
    nNA <- countNA(x)
    tab <- table(as.numeric(is.na(x)))
    tabcomb <- as.matrix(as.integer(names(tab)))
  } else {
    if (!is.null(delimiter)) {
      tmp <- grep(delimiter, colnames(x))
      if (length(tmp) > 0) {
        imp_var <- x[, tmp, drop = FALSE]
        x <- x[, -tmp, drop = FALSE]
        if (ncol(x) == 0)
          stop("Only the missing-index is given")
        if (is.matrix(imp_var) && range(imp_var) == c(0, 1)) {
          imp_var <- apply(imp_var, 2, as.logical)
        }

        if (is.null(dim(imp_var))) {
          if (!is.logical(imp_var)) {
            stop("The missing-index of imputed variables must be of the type logical")
          }
        } else {
          if (!any(as.logical(lapply(imp_var, is.logical)))) {
            stop("The missing-index of imputed variables must be of the type logical")
          }
        }
        imputed <- TRUE
      } else {
        warning("'delimiter' is given, but no missing-index variable is found",
                call. = FALSE)
      }
    }
    n <- nrow(x)
    cn <- colnames(x)
    if (is.null(cn)) {
      cn <- defaultNames(ncol(x))
    }

    nNA <- apply(x, 2, countNA)
    imp <- rep(FALSE, ncol(x))
    if (imputed) {
      nNA_imp <- countImp(x, delimiter, imp_var)
      imp <- nNA_imp > 0
      nNA <- nNA + nNA_imp
    }
    tmp <- ifelse(is.na(x), 1, 0)
    if (imputed) {
      tmp_imp <- isImp(x, pos = NULL, delimiter = delimiter,
                       imp_var = imp_var, selection = "none")[["missh"]]
      tmp_imp <- ifelse(tmp_imp, 2, 0)
      tmp[, colnames(tmp_imp)] <- tmp_imp
    }
    tab <- table(apply(tmp, 1, paste, collapse = ":"))
    tabcomb <- sapply(names(tab),
                      function(x) as.integer(unlist(strsplit(x, ":", fixed = TRUE))), USE.NAMES = FALSE)
    tabcomb <- if (is.null(dim(tabcomb))) {
      as.matrix(tabcomb)
    } else {
      t(tabcomb)
    }
  }
  miss <- data.frame(Variable = cn, Count = nNA, stringsAsFactors = FALSE)
  count <- as.integer(tab)
  res <- list(x = x, combinations = names(tab), count = count,
              percent = count * 100/n, missings = miss, tabcomb = tabcomb,
              imputed = imp)
  return(res)
}
